﻿using FluentValidation;
using FluentValidation.Validators;

namespace InternshipSystem.Core.Validators
{
    public class InternshipRegistrationValidator : AbstractValidator<InternshipRegistration>
    {
        public InternshipRegistrationValidator(Edition edition)
        {
            RuleFor(ir => ir.Start)
                .GreaterThanOrEqualTo(edition.EditionStart)
                .LessThanOrEqualTo(edition.EditionFinish);
            
            RuleFor(ir => ir.End)
                .GreaterThanOrEqualTo(ir => ir.Start)
                .LessThanOrEqualTo(edition.EditionFinish);

            RuleFor(ir => ir.Company)
                .SetValidator(new CompanyValidator());

            RuleFor(ir => ir.BranchAddress)
                .SetValidator(new BranchAddressValidator());

        }
    }

    public class BranchAddressValidator : AbstractValidator<BranchOffice>
    {
    }

    public class CompanyValidator : AbstractValidator<Company>
    {
    }
}